@@ -44,10 +44,13 @@ import shutil
 import subprocess
 import sys
 import tempfile
+import traceback
 import urllib2
 import webbrowser
 import yaml
 
+import bloom
+
 from bloom.commands.git.config import convert_old_bloom_conf
 from bloom.commands.git.config import edit as edit_track_cmd
 from bloom.commands.git.config import new as new_track_cmd
@@ -61,6 +64,7 @@ from bloom.git import inbranch
 from bloom.git import ls_tree
 from bloom.git import track_branches
 
+from bloom.logging import debug
 from bloom.logging import error
 from bloom.logging import fmt
 from bloom.logging import get_error_prefix
@@ -68,13 +72,16 @@ from bloom.logging import get_success_prefix
 from bloom.logging import info
 from bloom.logging import warning
 
+from bloom.util import add_global_arguments
 from bloom.util import change_directory
 from bloom.util import check_output
+from bloom.util import handle_global_arguments
 from bloom.util import maybe_continue
 
 try:
     from vcstools.vcs_abstraction import get_vcs_client
 except ImportError:
+    debug(traceback.format_exc())
     error("vcstools was not detected, please install it.", file=sys.stderr,
           exit=True)
 
@@ -175,6 +182,7 @@ def generate_ros_distro_diff(track, repository, distro, distro_file_url, distro_
         try:
             from catkin_pkg.packages import find_packages
         except ImportError:
+            debug(traceback.format_exc())
             error("catkin_pkg was not detected, please install it.",
                   file=sys.stderr, exit=True)
         packages = find_packages(os.getcwd())
@@ -190,7 +198,7 @@ def generate_ros_distro_diff(track, repository, distro, distro_file_url, distro_
         if packages and (len(packages) > 1 or packages.keys()[0] != '.'):
             distro_file['repositories'][repository]['packages'] = {}
             for path, package in packages.iteritems():
-                if os.path.dirname(path) == package.name:
+                if os.path.basename(path) == package.name:
                     distro_file['repositories'][repository]['packages'][package.name] = None
                 else:
                     distro_file['repositories'][repository]['packages'][package.name] = path
@@ -267,13 +275,13 @@ def create_fork(org, repo, user, password):
         error("Failed to create fork: {0} {1}".format(resp.status, resp.reason), exit=True)
 
 
-def create_pull_request(org, repo, user, password, base_branch, head_branch, title):
+def create_pull_request(org, repo, user, password, base_branch, head_branch, title, body=""):
     headers = {}
     headers["Authorization"] = "Basic {0}".format(base64.b64encode('{0}:{1}'.format(user, password)))
     conn = httplib.HTTPSConnection('api.github.com')
     data = {
         'title': title,
-        'body': "",
+        'body': body,
         'head': "{0}:{1}".format(user, head_branch),
         'base': base_branch
     }
@@ -291,6 +299,10 @@ def open_pull_request(track, repository, distro, distro_file_url=ROS_DISTRO_FILE
     distro_file_url = distro_file_url.format(distro)
     distro_file_raw = fetch_distro_file(distro_file_url)
     distro_file = yaml.load(distro_file_raw)
+    if repository in distro_file['repositories']:
+        orig_version = distro_file['repositories'][repository].get('version', None)
+    else:
+        orig_version = None
     udiff_patch_file, updated_distro_file = generate_ros_distro_diff(track, repository, distro,
                                                                      distro_file_url, distro_file,
                                                                      distro_file_raw)
@@ -316,10 +328,10 @@ def open_pull_request(track, repository, distro, distro_file_url=ROS_DISTRO_FILE
         if 'github.com' in host:
             gh_username = netrc_hosts[host][0]
             gh_password = netrc_hosts[host][2]
-        if None in [gh_username, gh_password]:
-            error("Either github username or github password is not set in the netrc.")
-            warning("Skipping the pull request...")
-            return
+    if None in [gh_username, gh_password]:
+        error("Either the github username or github password is not set in the ~/.netrc file.")
+        warning("Skipping the pull request...")
+        return
     # Check for fork
     info(fmt("@{bf}@!==> @|@!Checking for rosdistro fork on github..."))
     gh_user_repos = fetch_github_api('https://api.github.com/users/{0}/repos'.format(gh_username))
@@ -339,7 +351,14 @@ def open_pull_request(track, repository, distro, distro_file_url=ROS_DISTRO_FILE
     info(fmt("@{bf}@!==> @|@!" + "Cloning {0}/{1}...".format(gh_username, gh_repo)))
     temp_dir = tempfile.mkdtemp()
     new_branch = None
-    title = "{0}: update version to {1} in {2} [bloom]".format(repository, version, gh_path)
+    title = "{0}: {1} in '{2}' [bloom]".format(repository, version, os.path.basename(gh_path))
+    body = """\
+Increasing version of package(s) in repository `{0}`:
+- previous version: `{1}`
+- new version: `{2}`
+- distro file: `{3}`
+- bloom version: `{4}`
+""".format(repository, orig_version or 'null', version, gh_path, bloom.__version__)
     with change_directory(temp_dir):
         def _my_run(cmd):
             info(fmt("@{bf}@!==> @|@!" + str(cmd)))
@@ -358,6 +377,18 @@ def open_pull_request(track, repository, distro, distro_file_url=ROS_DISTRO_FILE
             while new_branch.format(count) in branches:
                 count += 1
             new_branch = new_branch.format(count)
+            # Final check
+            info(fmt("@{cf}Pull Request Title: @{yf}" + title))
+            info(fmt("@{cf}Pull Request Body : \n@{yf}" + body))
+            msg = fmt("@!Open a @|@{cf}pull request@| @!@{kf}from@| @!'@|@!@{bf}" +
+                      "{gh_username}/{gh_repo}:{new_branch}".format(**locals()) +
+                      "@|@!' @!@{kf}into@| @!'@|@!@{bf}" +
+                      "{gh_org}/{gh_repo}:{gh_branch}".format(**locals()) +
+                      "@|@!'?")
+            info(msg)
+            if not maybe_continue():
+                warning("Skipping the pull request...")
+                return
             _my_run('git checkout -b {0} bloom/{1}'.format(new_branch, gh_branch))
             with open('{0}'.format(gh_path), 'w') as f:
                 info(fmt("@{bf}@!==> @|@!Writing new distribution file: ") + str(gh_path))
@@ -365,18 +396,8 @@ def open_pull_request(track, repository, distro, distro_file_url=ROS_DISTRO_FILE
             _my_run('git add {0}'.format(gh_path))
             _my_run('git commit -m "{0}"'.format(title))
             _my_run('git push origin {0}'.format(new_branch))
-    # Final check
-    msg = fmt("@!Open a @{cf}pull request@| @!@{kf}from@| @!'@|@{bf}" +
-              "{gh_username}/{gh_repo}:{new_branch}".format(**locals()) +
-              "@|@!' @!@{kf}into@| @!'@|@{bf}" +
-              "{gh_org}/{gh_repo}:{gh_branch}".format(**locals()) +
-              "@|@!'?")
-    info(msg)
-    if not maybe_continue():
-        warning("Skipping the pull request...")
-        return
     # Open the pull request
-    return create_pull_request(gh_org, gh_repo, gh_username, gh_password, gh_branch, new_branch, title)
+    return create_pull_request(gh_org, gh_repo, gh_username, gh_password, gh_branch, new_branch, title, body)
 
 
 def perform_release(repository, track, distro, new_track, interactive, pretend):
@@ -510,9 +531,12 @@ def perform_release(repository, track, distro, new_track, interactive, pretend):
                 info(fmt(_success) + "Pull request opened at: '{0}'".format(pull_request_url))
                 webbrowser.open(pull_request_url)
             else:
+                info("The release of your packages was successful, but the pull request failed.")
+                info("Please manually open a pull request by editing the file here: '{0}'"
+                     .format(ROS_DISTRO_FILE).format(distro))
                 info(fmt(_error) + "No pull request opened.")
         except Exception as e:
-            error("Failed to open pull request: {0}".format(e), exit=True)
+            error("Failed to open pull request: {0} - {1}".format(type(e), e), exit=True)
 
 
 def get_argument_parser():
@@ -535,7 +559,9 @@ _quiet = False
 
 def main(sysargs=None):
     parser = get_argument_parser()
+    parser = add_global_arguments(parser)
     args = parser.parse_args(sysargs)
+    handle_global_arguments(args)
 
     if args.list_tracks:
         list_tracks(args.repository, args.ros_distro)
